@@ -1,6 +1,6 @@
 /**************************************************************************
 * This file is part of the Fraqtive program
-* Copyright (C) 2004-2008 Michał Męciński
+* Copyright (C) 2004-2009 Michał Męciński
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
 #include <QImageWriter>
 #include <QFileDialog>
 #include <QClipboard>
+#include <QProgressDialog>
 
 #include "tutorialdialog.h"
 #include "datastructures.h"
@@ -30,6 +31,8 @@
 #include "savebookmarkdialog.h"
 #include "loadpresetdialog.h"
 #include "savepresetdialog.h"
+#include "generateimagedialog.h"
+#include "imagegenerator.h"
 
 FraqtiveMainWindow::FraqtiveMainWindow() :
     m_tutorialDialog( NULL )
@@ -96,7 +99,7 @@ FraqtiveMainWindow::FraqtiveMainWindow() :
         restoreState( config->value( "State" ).toByteArray(), 1 );
 
     if ( config->value( "ShowTutorial", true ).toBool() )
-        on_actionQuickTutorial_activated();
+        on_actionQuickTutorial_triggered();
 }
 
 FraqtiveMainWindow::~FraqtiveMainWindow()
@@ -136,7 +139,7 @@ bool FraqtiveMainWindow::eventFilter( QObject* watched, QEvent* e )
     return QMainWindow::eventFilter( watched, e );
 }
 
-void FraqtiveMainWindow::on_actionQuit_activated()
+void FraqtiveMainWindow::on_actionQuit_triggered()
 {
     if ( isFullScreenMode() )
         leaveFullScreenMode();
@@ -144,7 +147,7 @@ void FraqtiveMainWindow::on_actionQuit_activated()
     close();
 }
 
-void FraqtiveMainWindow::on_actionFractalType_activated()
+void FraqtiveMainWindow::on_actionFractalType_triggered()
 {
     FractalTypeDialog dialog( this, m_model );
 
@@ -152,7 +155,7 @@ void FraqtiveMainWindow::on_actionFractalType_activated()
         m_model->setParameters( dialog.fractalType(), dialog.position() );
 }
 
-void FraqtiveMainWindow::on_actionEditGradient_activated()
+void FraqtiveMainWindow::on_actionEditGradient_triggered()
 {
     GradientDialog dialog( this );
     dialog.setGradient( m_model->gradient() );
@@ -167,7 +170,7 @@ void FraqtiveMainWindow::applyGradient( const Gradient& gradient )
     m_model->setGradient( gradient );
 }
 
-void FraqtiveMainWindow::on_actionFullScreen_activated()
+void FraqtiveMainWindow::on_actionFullScreen_triggered()
 {
     if ( isFullScreenMode() )
         leaveFullScreenMode();
@@ -199,7 +202,7 @@ void FraqtiveMainWindow::leaveFullScreenMode()
     show();
 }
 
-void FraqtiveMainWindow::on_actionDefaultPosition_activated()
+void FraqtiveMainWindow::on_actionDefaultPosition_triggered()
 {
     m_model->setDefaultPosition();
 }
@@ -209,12 +212,12 @@ void FraqtiveMainWindow::positionChanged()
     m_ui.actionDefaultPosition->setEnabled( !m_model->hasDefaultPosition() );
 }
 
-void FraqtiveMainWindow::on_actionNavigateBack_activated()
+void FraqtiveMainWindow::on_actionNavigateBack_triggered()
 {
     m_model->navigateBackward();
 }
 
-void FraqtiveMainWindow::on_actionNavigateForward_activated()
+void FraqtiveMainWindow::on_actionNavigateForward_triggered()
 {
     m_model->navigateForward();
 }
@@ -225,35 +228,56 @@ void FraqtiveMainWindow::navigationChanged()
     m_ui.actionNavigateForward->setEnabled( m_model->canNavigateForward() );
 }
 
-void FraqtiveMainWindow::on_actionLoadPreset_activated()
+void FraqtiveMainWindow::on_actionLoadPreset_triggered()
 {
     LoadPresetDialog dialog( this );
     dialog.setModel( m_model );
     dialog.exec();
 }
 
-void FraqtiveMainWindow::on_actionSavePreset_activated()
+void FraqtiveMainWindow::on_actionSavePreset_triggered()
 {
     SavePresetDialog dialog( this );
     dialog.setModel( m_model );
     dialog.exec();
 }
 
-void FraqtiveMainWindow::on_actionLoadBookmark_activated()
+void FraqtiveMainWindow::on_actionLoadBookmark_triggered()
 {
     LoadBookmarkDialog dialog( this );
     dialog.setModel( m_model );
     dialog.exec();
 }
 
-void FraqtiveMainWindow::on_actionSaveBookmark_activated()
+void FraqtiveMainWindow::on_actionSaveBookmark_triggered()
 {
     SaveBookmarkDialog dialog( this );
     dialog.setModel( m_model );
     dialog.exec();
 }
 
-void FraqtiveMainWindow::on_actionSaveImage_activated()
+void FraqtiveMainWindow::on_actionSaveImage_triggered()
+{
+    QImageWriter* writer = createImageWriter();
+    if ( writer ) {
+        QImage image = currentImage();
+
+        if ( !writer->write( image ) ) {
+            QMessageBox::warning( this, tr( "Error" ), tr( "The selected file could not be saved." ) );
+        }
+
+        delete writer;
+    }
+}
+
+void FraqtiveMainWindow::on_actionCopyImage_triggered()
+{
+    QImage image = currentImage();
+
+    QApplication::clipboard()->setImage( image );
+}
+
+QImageWriter* FraqtiveMainWindow::createImageWriter()
 {
     QList<QByteArray> supportedFormats = QImageWriter::supportedImageFormats();
 
@@ -302,24 +326,15 @@ void FraqtiveMainWindow::on_actionSaveImage_activated()
         config->setValue( "SaveFormat", format );
         config->setValue( "SavePath", QFileInfo( fileName ).absolutePath() );
 
-        QImageWriter writer( fileName, format );
+        QImageWriter* writer = new QImageWriter( fileName, format );
 
         if ( format == "tiff" )
-            writer.setCompression( 1 );
+            writer->setCompression( 1 );
 
-        QImage image = currentImage();
-
-        if ( !writer.write( image ) ) {
-            QMessageBox::warning( this, tr( "Error" ), tr( "The selected file could not be saved." ) );
-        }
+        return writer;
     }
-}
 
-void FraqtiveMainWindow::on_actionCopyImage_activated()
-{
-    QImage image = currentImage();
-
-    QApplication::clipboard()->setImage( image );
+    return NULL;
 }
 
 QImage FraqtiveMainWindow::currentImage()
@@ -331,7 +346,50 @@ QImage FraqtiveMainWindow::currentImage()
     return QImage();
 }
 
-void FraqtiveMainWindow::on_action2DView_activated()
+void FraqtiveMainWindow::on_actionGenerateImage_triggered()
+{
+    GenerateImageDialog dialog( this );
+
+    if ( dialog.exec() == QDialog::Accepted ) {
+        ImageGenerator generator( this );
+        if ( !generator.setResolution( dialog.resolution() ) ) {
+            QMessageBox::warning( this, tr( "Error" ), tr( "Not enough memory to generate image." ) );
+            return;
+        }
+        generator.setParameters( m_model->fractalType(), m_model->position() );
+        generator.setColorSettings( m_model->gradient(), m_model->backgroundColor(), m_model->colorMapping() );
+        generator.setGeneratorSettings( dialog.generatorSettings() );
+        generator.setViewSettings( dialog.viewSettings() );
+
+        QProgressDialog progress( this );
+        progress.setWindowModality( Qt::WindowModal );
+        progress.setRange( 0, generator.maximumProgress() );
+        progress.setWindowTitle( tr( "Generate Image" ) );
+        progress.setLabelText( tr( "Calculating..." ) );
+        progress.setValue( 0 );
+
+        QEventLoop eventLoop;
+
+        connect( &generator, SIGNAL( progressChanged( int ) ), &progress, SLOT( setValue( int ) ), Qt::QueuedConnection );
+        connect( &generator, SIGNAL( completed() ), &eventLoop, SLOT( quit() ), Qt::QueuedConnection );
+        connect( &progress, SIGNAL( canceled() ), &eventLoop, SLOT( quit() ) );
+
+        generator.start();
+        eventLoop.exec();
+
+        if ( !progress.wasCanceled() ) {
+            QImageWriter* writer = createImageWriter();
+            if ( writer ) {
+                if ( !writer->write( generator.image() ) ) {
+                    QMessageBox::warning( this, tr( "Error" ), tr( "The selected file could not be saved." ) );
+                }
+                delete writer;
+            }
+        }
+    }
+}
+
+void FraqtiveMainWindow::on_action2DView_triggered()
 {
     if ( !qobject_cast<ImageView*>( m_ui.mainContainer->view() ) ) {
         ImageView* view = new ImageView( m_ui.mainContainer, m_model->presenter() );
@@ -349,7 +407,7 @@ void FraqtiveMainWindow::on_action2DView_activated()
     m_ui.action3DView->setChecked( false );
 }
 
-void FraqtiveMainWindow::on_action3DView_activated()
+void FraqtiveMainWindow::on_action3DView_triggered()
 {
     if ( !qobject_cast<MeshView*>( m_ui.mainContainer->view() ) ) {
         MeshView* view = new MeshView( m_ui.mainContainer, m_model->presenter() );
@@ -368,7 +426,7 @@ void FraqtiveMainWindow::on_action3DView_activated()
     m_ui.action3DView->setChecked( true );
 }
 
-void FraqtiveMainWindow::on_actionQuickTutorial_activated()
+void FraqtiveMainWindow::on_actionQuickTutorial_triggered()
 {
     if ( !m_tutorialDialog )
         m_tutorialDialog = new TutorialDialog( this );
@@ -377,22 +435,22 @@ void FraqtiveMainWindow::on_actionQuickTutorial_activated()
     m_tutorialDialog->activateWindow();
 }
 
-void FraqtiveMainWindow::on_actionAboutFraqtive_activated()
+void FraqtiveMainWindow::on_actionAboutFraqtive_triggered()
 {
-    QString version = "0.4.4";
+    QString version = "0.4.5";
     QString link = "<a href=\"http://fraqtive.mimec.org\">fraqtive.mimec.org</a>";
 
     QString message;
     message += "<h3>" + tr( "Fraqtive %1" ).arg( version ) + "</h3>";
     message += "<p>" + tr( "A Mandelbrot family fractal generator." ) + "</p>";
-    message += "<p>" + trUtf8( "Copyright (C) 2004-2008 Michał Męciński" ) + "</p>";
+    message += "<p>" + trUtf8( "Copyright (C) 2004-2009 Michał Męciński" ) + "</p>";
     message += "<p>" + tr( "This program is licensed under the GNU General Public License." ) + "</p>";
     message += "<p>" + tr( "Visit %1 for more information." ).arg( link ) + "</p>";
 
     QMessageBox::about( this, tr( "About Fraqtive" ), message );
 }
 
-void FraqtiveMainWindow::on_actionAboutQt_activated()
+void FraqtiveMainWindow::on_actionAboutQt_triggered()
 {
     QMessageBox::aboutQt( this );
 }
