@@ -1,6 +1,6 @@
 /**************************************************************************
 * This file is part of the Fraqtive program
-* Copyright (C) 2004-2008 Michał Męciński
+* Copyright (C) 2004-2009 Michał Męciński
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,6 +10,13 @@
 
 #include "fraqtiveapplication.h"
 
+#ifdef HAVE_STATIC_JPEG
+Q_IMPORT_PLUGIN( qjpeg )
+#endif
+#ifdef HAVE_STATIC_TIFF
+Q_IMPORT_PLUGIN( qtiff )
+#endif
+
 int main( int argc, char* argv[] )
 {
     FraqtiveApplication application( argc, argv );
